//===========================================================
//  Filename: MyExp.cpp
//      3D studio MAX exporter plugin
//      Copyright 2003
//      Created by Roman E. Marchenko, vortex@library.ntu-kpi.kiev.ua
//      $Id$
//===========================================================

#include "MyExp.h"
#include <iparamb2.h>
#include <simpobj.h>

// Global variables
#define PB_RADIUS						0

static SceneSaver	MyTreeEnum;
static ofstream		fFile;
Interface			*ip;
TIMDFile* imd;
//-----------------------------------------------------------
MyExp::MyExp()
{
}
//-----------------------------------------------------------
MyExp::~MyExp()
{
}
static int Alert(char* s1, char* s2="IMD ", int option = MB_OK) 
{
	return MessageBox(GetActiveWindow(), s1, s2, option);
}
//-----------------------------------------------------------
int MyExp::DoExport(const TCHAR *name, ExpInterface *ei,
					Interface *i, BOOL suppressPromts, DWORD options)
{
	imd=new TIMDFile();
	// Globalize i var
	ip = i;
	// Go Through all scene nodes
	ei->theScene->EnumTree(&MyTreeEnum);
	if(imd->Save((char*)name))
	{			
		char str[500];
		sprintf(str,"    %s.   .",ExtractFileName(name).data());
		Alert(str);
	}
	delete imd;
	return 1;
}
//-----------------------------------------------------------
BOOL MyExp::SupportsOptions(int ext, DWORD options)
{
	return FALSE;
}
//-----------------------------------------------------------
//    Scene saver class
//-----------------------------------------------------------
int SceneSaver::callback(INode *node)
{
	ProcNode(node);
	return TREE_CONTINUE;
}
//-----------------------------------------------------------
// All node process here
void SceneSaver::ProcNode(INode *node)
{
	int Del;
	int mat_index;
	// Get TriObject from node
	ObjectState os=node->EvalWorldState(0);
	if(os.obj)
	{
		if(os.obj->SuperClassID()==HELPER_CLASS_ID) //  
		{
			ExportHelper(node);
			return;
		}
	}
	if(strcmp("GBND",node->GetName())==0)
	{
		ExportGBND(node);
		return;
	}
	if(strcmp("GELP",node->GetName())==0)
	{
		ExportGELP(node);
		return;
	}
	TriObject *TObj;
	TObj = GetTriObjFromNode(node, Del);
	if (!TObj) return;
	mat_index=ExportMaterial(node);
	ExportGVTX(TObj,mat_index,node->GetObjTMAfterWSM(ip->GetTime()));
}
//-----------------------------------------------------------
// One has one material. Save it here
int SceneSaver::ExportMaterial(INode *node)
{
	Mtl *m = node->GetMtl();
	int index=-1;
	if (!m) return 0;
	if (m->ClassID() != Class_ID(DMTL_CLASS_ID, 0)) return 0;
	StdMat *mStd = (StdMat *)m;
	TGMAT* material=(TGMAT*)imd->gmat->First();
	for(int i=0;i<imd->gmat->Count();i++)
	{
		if(strcmp(material->material_name,m->GetName())==0)
		{
			return i;
		}
		material=(TGMAT*)imd->gmat->Next();
	}
	index=imd->gmat->Count();
	material=new TGMAT(mStd->GetName());
	material->texture_count=0;
	for(int i=0;i<12;i++)
	{
		if(mStd->MapEnabled(i))
		{
			material->textures[material->texture_count].type=i;
			Texmap *tmap=mStd->GetSubTexmap(i); 
			BitmapTex *tex=(BitmapTex*)tmap;
			material->textures[material->texture_count].file_name=strdup(ExtractFileName(tex->GetMapName()).data());
			material->textures[material->texture_count].koeff=mStd->GetTexmapAmt(i,0);
			material->texture_count++;
		}
	}
	Color c;
	material->koeff.Opacity=mStd->GetOpacity(ip->GetTime());
	c=mStd->GetAmbient(ip->GetTime());
	material->koeff.Shininess=0;
	material->koeff.ShinStr=0;
	material->koeff.SelfIllum=1;
	material->koeff.OpacFalloff=1;
	material->koeff.WireSize=1;
	material->koeff.IOR=39.0;
	material->koeff.Ambient.x=c.r;
	material->koeff.Ambient.y=c.g;
	material->koeff.Ambient.z=c.b;
	c=mStd->GetDiffuse(ip->GetTime());
	material->koeff.Diffuse.x=c.r;
	material->koeff.Diffuse.y=c.g;
	material->koeff.Diffuse.z=c.b;
	c=mStd->GetSpecular(ip->GetTime());
	material->koeff.Specular.x=c.r;
	material->koeff.Specular.y=c.g;
	material->koeff.Specular.z=c.b;
	c=mStd->GetFilter(ip->GetTime());
	material->koeff.Filter.x=c.r;
	material->koeff.Filter.y=c.g;
	material->koeff.Filter.z=c.b;
	imd->gmat->Add(material);
	imd->gmat_count++;
	return index;
}
//-----------------------------------------------------------
void SceneSaver::ExportGVTX(TriObject *TObj,int mat_index,Matrix3 tm)
{
	TObj->mesh.buildNormals();
	TGVTX* vertex=new TGVTX(TObj->mesh.numFaces*3,TObj->mesh.numFaces,mat_index);
	for(int i=0;i<TObj->mesh.numFaces;i++)
	{
		for(int j=0;j<3;j++)
		{
			Point3 v=tm*TObj->mesh.verts[TObj->mesh.faces[i].v[j]];
			vertex->points[i*3+j].x=v.x;
			vertex->points[i*3+j].y=v.y;
			vertex->points[i*3+j].z=v.z;
			vertex->points[i*3+j].a=0.0;
			vertex->points[i*3+j].b=0.0;
			if(TObj->mesh.numTVerts!=0)
			{
				vertex->points[i*3+j].a=TObj->mesh.tVerts[TObj->mesh.tvFace[i].t[j]].x;
				vertex->points[i*3+j].b=1.0-TObj->mesh.tVerts[TObj->mesh.tvFace[i].t[j]].y;
			}
			v=TObj->mesh.getNormal(TObj->mesh.faces[i].v[j]);
			vertex->vnormals[i*3+j].x=v.x;
			vertex->vnormals[i*3+j].y=v.y;
			vertex->vnormals[i*3+j].z=v.z;
		}
		vertex->polygons[i].a=i*3;
		vertex->polygons[i].b=i*3+1;
		vertex->polygons[i].c=i*3+2;
		vertex->fnormals[j].x=0;
		vertex->fnormals[j].y=0;
		vertex->fnormals[j].z=0;
	}
	imd->gvtx->Add(vertex);
	imd->gvtx_count++;
}
//-----------------------------------------------------------
void SceneSaver::ExportHelper(INode* node)
{
	Matrix3 tm=node->GetObjTMAfterWSM(ip->GetTime());
	HelperObject* helper=(HelperObject*)node->EvalWorldState(ip->GetTime()).obj;
	Point3 a1;
	Matrix3 tm1;
	tm1.IdentityMatrix();
	Point3 a,b,c,d;
	float size;
	IParamBlock2* pb=(IParamBlock2*)helper->GetParamBlock();
	Interval ivalid=FOREVER;
	pb->GetValue(pointobj_size,ip->GetTime(),size,ivalid);
	a.x=-0.5;
	a.y=-0.5;
	a.z=-0.5;
	b.x=-0.5;
	b.y=-0.5;
	b.z=0.5;
	c.x=-0.5;
	c.y=0.5;
	c.z=-0.5;
	d.x=0.5;
	d.y=-0.5;
	d.z=-0.5;
	a=tm*a;
	b=tm*b;
	c=tm*c;
	d=tm*d;
	float x,y,z; //   
	x=sqrt((a.x-d.x)*(a.x-d.x)+(a.y-d.y)*(a.y-d.y)+(a.z-d.z)*(a.z-d.z));
	y=sqrt((a.x-c.x)*(a.x-c.x)+(a.y-c.y)*(a.y-c.y)+(a.z-c.z)*(a.z-c.z));
	z=sqrt((a.x-b.x)*(a.x-b.x)+(a.y-b.y)*(a.y-b.y)+(a.z-b.z)*(a.z-b.z));
	a1.x=0;
	a1.y=-0.25*size;
	a1.z=0;
	tm1.Translate(a1);
	a.x=1.0/x;
	a.y=1.0/y;
	a.z=1.0/z;
	tm1.Scale(a);
	tm1=tm1*tm;
	TREFPoint* refs=new TREFPoint(node->GetName());
	refs->x=x/2*size;
	refs->y=y/2*size;
	refs->z=z/2*size;
	refs->matrix[0][0]=tm1.GetRow(0)[0];
	refs->matrix[0][1]=tm1.GetRow(0)[1];
	refs->matrix[0][2]=tm1.GetRow(0)[2];
	refs->matrix[3][0]=tm1.GetRow(3)[0];
	refs->matrix[1][0]=tm1.GetRow(1)[0];
	refs->matrix[1][1]=tm1.GetRow(1)[1];
	refs->matrix[1][2]=tm1.GetRow(1)[2];
	refs->matrix[3][1]=tm1.GetRow(3)[1];
	refs->matrix[2][0]=tm1.GetRow(2)[0];
	refs->matrix[2][1]=tm1.GetRow(2)[1];
	refs->matrix[2][2]=tm1.GetRow(2)[2];
	refs->matrix[3][2]=tm1.GetRow(3)[2];
	refs->matrix[0][3]=0;
	refs->matrix[1][3]=0;
	refs->matrix[2][3]=0;
	refs->matrix[3][3]=0;
	imd->gref->refs->Add(refs);
}
//-----------------------------------------------------------
void SceneSaver::ExportGBND(INode* node)
{
	Matrix3 tm=node->GetObjTMAfterWSM(ip->GetTime());
	float radius;
	Interval ivalid=FOREVER;
	GenSphere* sphere=(GenSphere*)node->EvalWorldState(ip->GetTime()).obj;
	IParamBlock* pb=(IParamBlock*)sphere->GetParamBlock();
	pb->GetValue(PB_RADIUS,ip->GetTime(),radius,ivalid);
	Point3 a,b;
	a.x=a.y=a.z=b.x=b.y=0;
	b.z=radius;
	a=tm*a;
	b=tm*b;
	radius=sqrt((a.x-b.x)*(a.x-b.x)+(a.y-b.y)*(a.y-b.y)+(a.z-b.z)*(a.z-b.z));
	if(imd->gbnd->mainsphera->r==0) //      ...
	{//        ...
		imd->gbnd->mainsphera->x=a.x;
		imd->gbnd->mainsphera->y=a.y;
		imd->gbnd->mainsphera->z=a.z;
		imd->gbnd->mainsphera->r=radius;
	}else
	{
		TSphera* s=new TSphera(a.x,a.y,a.z,radius);
		if(imd->gbnd->mainsphera->r<radius) //    ,  
		{//      ,     
			imd->gbnd->spheres->Add(imd->gbnd->mainsphera);
			imd->gbnd->mainsphera=s;
		}else
		{
			imd->gbnd->spheres->Add(s); //     ...
		}
	}
}
//-----------------------------------------------------------
void SceneSaver::ExportGELP(INode* node)
{
	Matrix3 tm=node->GetObjTMAfterWSM(ip->GetTime());
	float radius;
	Interval ivalid=FOREVER;
	GenSphere* sphere=(GenSphere*)node->EvalWorldState(ip->GetTime()).obj;
	IParamBlock* pb=(IParamBlock*)sphere->GetParamBlock();
	pb->GetValue(PB_RADIUS,ip->GetTime(),radius,ivalid);
	Point3 a,b,c,d;
	float x,y,z;
	a.x=a.y=a.z=0;
	b.x=radius;
	b.y=0;
	b.z=0;
	c.x=0;
	c.y=radius;
	c.z=0;
	d.x=0;
	d.y=0;
	d.z=radius;
	a=tm*a;
	b=tm*b;
	c=tm*c;
	d=tm*d;
	x=sqrt((a.x-b.x)*(a.x-b.x)+(a.y-b.y)*(a.y-b.y)+(a.z-b.z)*(a.z-b.z));
	y=sqrt((a.x-c.x)*(a.x-c.x)+(a.y-c.y)*(a.y-c.y)+(a.z-c.z)*(a.z-c.z));
	z=sqrt((a.x-d.x)*(a.x-d.x)+(a.y-d.y)*(a.y-d.y)+(a.z-d.z)*(a.z-d.z));
	if(imd->gelp!=0)delete imd->gelp;
	imd->gelp=new TGELP(a.x,a.y,a.z,x,y,z);
}
//-----------------------------------------------------------

//-----------------------------------------------------------
//    Standalone functions
//-----------------------------------------------------------

TriObject *GetTriObjFromNode(INode *node, int &deleteIt)
{
	deleteIt = FALSE;
	Object *obj = node->EvalWorldState(ip->GetTime()).obj;
	if (obj->CanConvertToType(Class_ID(TRIOBJ_CLASS_ID,0)))
	{
		TriObject *tri = (TriObject *) obj->ConvertToType(ip->GetTime(), 
			Class_ID(TRIOBJ_CLASS_ID, 0));
		if (obj != tri) deleteIt = TRUE;
		return tri;
	}
	else return NULL;
}

remString ExtractFileName(remString filename)
{
        if (filename.size() == 0) return "";

        int i = filename.size();
        remString buf;
        while((filename[i] != '\\') && (i > 0))
        {
                buf = filename[i--] + buf;
        }
        return buf;
}