#ifndef __IMDFILE__
#define __IMDFILE__
#include <max.h>
#include <tab.h>
#include <stdio.h>
#include <string.h>
//---------------------------------------------------------------------------
class TBaseObject
{
public:
	TBaseObject(){;};
	~TBaseObject(){;};
};
//---------------------------------------------------------------------------
class TObjectNode
{
	private:
	TBaseObject* data;
	unsigned int index;
	TObjectNode* next,*prev;
	public:
	TObjectNode(TBaseObject* a);
	~TObjectNode();
	TObjectNode* Next();
	TObjectNode* Prev();
	TBaseObject* Data();
	friend class TObjectList;
};
//---------------------------------------------------------------------------
class TObjectList
{
	private:
	TObjectNode* first,*win;
	unsigned int count;
	public:
	TObjectList();
	~TObjectList();
	void Add(TBaseObject* a);
	TBaseObject* Next();
	TBaseObject* Prev();
	TBaseObject* First();
	int Count();
	friend class TIMDFile;
};
//---------------------------------------------------------------------------
class TBlockObject:public TBaseObject
{
public:
	char sign[5];
	unsigned int size;
	TBlockObject(char* asign):TBaseObject(){memcpy(sign,asign,4);sign[4]=(char)0;};
	~TBlockObject(){;};
};
//---------------------------------------------------------------------------
typedef struct point
{
  float x,y,z;
  float a,b;
}MPoint;
//---------------------------------------------------------------------------
typedef struct polygon
{
  unsigned short int a,b,c;
}MPolygon;
//---------------------------------------------------------------------------
typedef struct vector
{
  float x,y,z;
}TVector;
//---------------------------------------------------------------------------
class TGVTX:public TBlockObject
{
public:
  unsigned int point_count,polygon_count,material_index;
  MPoint* points;
  MPolygon* polygons;
  TVector* vnormals;
  TVector* fnormals;
  TGVTX(unsigned int apoint_count,unsigned int apolygon_count,
	  unsigned int amaterial_index);
  ~TGVTX();
};
//---------------------------------------------------------------------------
typedef struct tgmatkoeff
{
  float Shininess;		
  float ShinStr;		
  TVector Diffuse;		
  float SelfIllum;		
  TVector Ambient;		
  float Opacity;		
  TVector Specular;
  float OpacFalloff;		
  TVector Filter;
  float WireSize;
  float IOR;
}TGMATKoeff;
//---------------------------------------------------------------------------
typedef struct ttexture
{
	int type;
	char* file_name;
	float koeff;
}TTexture;
//---------------------------------------------------------------------------
class TGMAT:public TBlockObject
{
public:
  char* material_name;
  TGMATKoeff koeff;
  unsigned int texture_count;
  TTexture textures[12];
  TGMAT(char* amaterial_name);
  ~TGMAT();
};
class TREFPoint:public TBaseObject
{
public:
	char* name;
	float matrix[4][4];
	float x,y,z;
	TREFPoint(char* aname);
	~TREFPoint();
};
class TGREF:public TBlockObject
{
public:
//    int ref_count;
	TObjectList* refs;
	TGREF();
	~TGREF();
//	TREFPoint refs[100];
};
class TSphera:public TBaseObject
{
public:
	float x,y,z,r;
	TSphera(float ax,float ay,float az,float ar);
	~TSphera();
};
class TGBND:public TBlockObject
{
public:
	TSphera* mainsphera;
	int count;
	TObjectList* spheres;
	TGBND();
	~TGBND();
//	TSphera spheres[100];
};
class TGELP:public TBlockObject
{
public:
	float x,y,z,dx,dy,dz;
	TGELP(float ax,float ay,float az,float adx,float ady,float adz);
	~TGELP();
};
//---------------------------------------------------------------------------
class TIMDFile
{
  public:
    char* buf;
    int size;
    int index;
	TObjectList* gmat;
	TObjectList* gvtx;
//	TGMAT gmat[20];
//	TGVTX gvtx[50];
	TGREF* gref;
	TGBND* gbnd;
	TGELP* gelp;
    int gvtx_count;
    int gmat_count;
    TIMDFile();
    ~TIMDFile();
    int Save(char* file_name);
/*	void ExportNode();
    TGVTX* GetGVTX(int i);
    TGMAT* GetGMAT(int i);
	TGREF* GetGREF();
	TGBND* GetGBND();
	TGELP* GetGELP();
	int AllVertexCount();
	int AllFacesCount();*/
};
//---------------------------------------------------------------------------
#endif