#include "imdfile.h"
#include <stdio.h>
#include <string.h>

//---------------------------------------------------------------------------
TIMDFile::TIMDFile()
{
  gmat=0;
  gvtx=0;
  gvtx_count=0;
  gmat_count=0;
  buf=0;
  size=0;
  gelp=0;
}
//---------------------------------------------------------------------------
TIMDFile::~TIMDFile()
{
  if(gmat!=0)
  {
	for(int i=0;i<gmat_count;i++)
	{
	  delete[] gmat[i].material_name;
	  for(int j=0;j<gmat[i].texture_count;j++)
		  delete[] gmat[i].textures[j].file_name;
	  delete[] gmat[i].textures;
	}
	delete[] gmat;
  }
  if(gvtx!=0)
  {
	  for(int i=0;i<gvtx_count;i++)
	  {
		  delete[] gvtx[i].points;
		  delete[] gvtx[i].polygons;
          delete[] gvtx[i].normals;
		  delete[] gvtx[i].texture_vertex;
	  }
	  delete[] gvtx;
  }
  if(gbnd.spheres!=0)delete[] gbnd.spheres;
  if(gelp!=0)delete gelp;
  for(int i=0;i<gref.ref_count;i++)
	  delete gref.refs[i].name;
  delete[] gref.refs;
  if(buf!=0)
  {
	delete[] buf;
  }
}
//---------------------------------------------------------------------------
void TIMDFile::Open(char* file_name)
{
  FILE* F=fopen(file_name,"rb");
  int j,k;
  char sign[5];
  char block_sign[5];
  unsigned int block_size;
  int tmpsize;
  fread(sign,4,1,F);
  fread(&size,4,1,F);
  sign[4]=(char)0;
  if(strcmp(sign,"LIBF")!=0)
  {
    fclose(F);
//    exit;
  }
  size-=8;      
  buf=new char[size];
  fread(buf,size,1,F);
  fclose(F);
  block_sign[4]=(char)0;
  memcpy(&gvtx_count,&buf[0x30-4],4);
  memcpy(&gmat_count,&buf[0x30],4);
  gvtx=new TGVTX[gvtx_count];
  gmat=new TGMAT[gmat_count];
  for(index=52,j=0,k=0;index<size;index+=block_size)
  {
        memcpy(block_sign,&buf[index],4);
        memcpy(&block_size,&buf[index+4],4);
        if(strcmp(block_sign,"GVTX")==0)
        {
            memcpy(gvtx[j].sign,block_sign,5);
            gvtx[j].length=block_size;
            memcpy(&gvtx[j].point_count,&buf[index+20],4);
            memcpy(&gvtx[j].polygon_count,&buf[index+24],4);
			memcpy(&gvtx[j].material_index,&buf[index+12],4);
            gvtx[j].points=new MPoint[gvtx[j].point_count];
            gvtx[j].polygons=new MPolygon[gvtx[j].polygon_count];
            gvtx[j].normals=new TVector[gvtx[j].point_count];
            gvtx[j].texture_vertex=new TVector[gvtx[j].polygon_count];
            memcpy(gvtx[j].points,&buf[index+28],gvtx[j].point_count*20);
            memcpy(gvtx[j].polygons,&buf[index+28+gvtx[j].point_count*20],gvtx[j].polygon_count*6);
            memcpy(gvtx[j].normals,&buf[index+28+gvtx[j].point_count*20+gvtx[j].polygon_count*6],
                        gvtx[j].point_count*12);
            memcpy(gvtx[j].texture_vertex,&buf[index+28+gvtx[j].point_count*20+
                        gvtx[j].polygon_count*6+gvtx[j].point_count*12],
                        gvtx[j].polygon_count*12);
            j++;
        }
        if(strcmp(block_sign,"GMAT")==0)
        {
			memcpy(gmat[k].sign,block_sign,5);
			gmat[k].length=block_size;
			memcpy(&tmpsize,&buf[index+12],4);
			gmat[k].material_name=new char[tmpsize];
			memcpy(gmat[k].material_name,&buf[index+16],tmpsize);
			memcpy(&gmat[k].koeff,&buf[index+16+tmpsize],sizeof(TGMATKoeff));
			memcpy(&gmat[k].texture_count,&buf[index+16+tmpsize+sizeof(TGMATKoeff)],4);
//			if(gmat[k].texture_count>0)
			{
				gmat[k].textures=new TTexture[gmat[k].texture_count];
				int dx=index+20+tmpsize+sizeof(TGMATKoeff);
				for(int ij=0;ij<gmat[k].texture_count;ij++)
				{
					memcpy(&gmat[k].textures[ij].type,&buf[dx],4);
					dx+=4;
					memcpy(&tmpsize,&buf[dx],4);
					dx+=4;
					gmat[k].textures[ij].file_name=new char[tmpsize];
					memcpy(gmat[k].textures[ij].file_name,&buf[dx],tmpsize);
					dx+=tmpsize;
					memcpy(&gmat[k].textures[ij].koeff,&buf[dx],4);
					dx+=4;
				}
			}
			k++;
        }
        if(strcmp(block_sign,"GREF")==0)
        {
			memcpy(gref.sign,block_sign,5);
			gref.length=block_size;
			memcpy(&gref.ref_count,&buf[index+12],4);
//			if(gref.ref_count>0)
			{
				gref.refs=new TREFPoint[gref.ref_count];
				int tmp=index+16;
				for(int ij=0;ij<gref.ref_count;ij++)
				{
					int l;
					memcpy(&l,&buf[tmp],4);
					gref.refs[ij].name=new char[l];
					memcpy(gref.refs[ij].name,&buf[tmp+4],l);
					for(int i1=0;i1<4;i1++)
						for(int j1=0;j1<4;j1++)
							memcpy(&gref.refs[ij].matrix[i1][j1],&buf[tmp+l+4+i1*16+j1*4],4);
					memcpy(&gref.refs[ij].x,&buf[tmp+l+17*4],4);
					memcpy(&gref.refs[ij].y,&buf[tmp+l+18*4],4);
					memcpy(&gref.refs[ij].z,&buf[tmp+l+19*4],4);
					tmp=tmp+l+21*4;
				}
			}
		}
        if(strcmp(block_sign,"GBND")==0)
        {
			memcpy(gbnd.sign,block_sign,5);
			gbnd.length=block_size;
			memcpy(&gbnd.mainsphera,&buf[index+12],16);
			memcpy(&gbnd.count,&buf[index+28],4);
//			if(gbnd.count>0)
			{
				gbnd.spheres=new TSphera[gbnd.count];
				memcpy(gbnd.spheres,&buf[index+32],16*gbnd.count);
			}
		}
        if(strcmp(block_sign,"GELP")==0)
        {
			gelp=new TGELP;
			memcpy(gelp->sign,block_sign,5);
			gelp->length=block_size;
			memcpy(&gelp->x,&buf[index+12],6*4);
		}
 }
}
//---------------------------------------------------------------------------
TGVTX* TIMDFile::GetGVTX(int i)
{
  if((i>=0)&&(i<gvtx_count))return &gvtx[i];
  return 0;
}
//---------------------------------------------------------------------------
TGMAT* TIMDFile::GetGMAT(int i)
{
  if((i>=0)&&(i<gmat_count))return &gmat[i];
  return 0;
}
//---------------------------------------------------------------------------
TGREF* TIMDFile::GetGREF()
{
	return &gref;
}
//---------------------------------------------------------------------------
TGBND* TIMDFile::GetGBND()
{
	return &gbnd;
}
//---------------------------------------------------------------------------
TGELP* TIMDFile::GetGELP()
{
	return gelp;
}
//---------------------------------------------------------------------------
int TIMDFile::AllVertexCount()
{
	int result=0;
	for(int i=0;i<gvtx_count;i++)
		result+=gvtx[i].point_count;
	return result;
}
//---------------------------------------------------------------------------
int TIMDFile::AllFacesCount()
{
	int result=0;
	for(int i=0;i<gvtx_count;i++)
		result+=gvtx[i].polygon_count;
	return result;
}
//---------------------------------------------------------------------------
