#ifndef __IMDFILE__
#define __IMDFILE__
//---------------------------------------------------------------------------
typedef struct point
{
  float x,y,z;
  float a,b;
}MPoint;
//---------------------------------------------------------------------------
typedef struct polygon
{
  unsigned short int a,b,c;
}MPolygon;
//---------------------------------------------------------------------------
typedef struct vector
{
  float x,y,z;
}TVector;
//---------------------------------------------------------------------------
typedef struct tgvtx
{
  char sign[5];
  unsigned int length,point_count,polygon_count,material_index;
  MPoint* points;
  MPolygon* polygons;
  TVector* normals;
  TVector* texture_vertex;
}TGVTX;
//---------------------------------------------------------------------------
typedef struct tgmatkoeff
{
  float Shininess;		
  float ShinStr;		
  TVector Diffuse;		
  float SelfIllum;		
  TVector Ambient;		
  float Opacity;		
  TVector Specular;
  float OpacFalloff;		
  TVector Filter;
  float WireSize;
  float IOR;
}TGMATKoeff;
//---------------------------------------------------------------------------
typedef struct ttexture
{
	int type;
	char* file_name;
	float koeff;
}TTexture;
//---------------------------------------------------------------------------
typedef struct tgmat
{
  char sign[5];
  unsigned int length;
  char* material_name;
  TGMATKoeff koeff;
  int texture_count;
  TTexture* textures;
}TGMAT;
typedef struct trefpoint
{
	char* name;
	float matrix[4][4];
	float x,y,z;
}TREFPoint;
typedef struct tgref
{
	char sign[5];
    int length,ref_count;
    TREFPoint *refs;
}TGREF;
typedef struct tsphera
{
	float x,y,z,r;
}TSphera;
typedef struct tgbnd
{
	char sign[5];
	int length;
	TSphera mainsphera;
	int count;
	TSphera* spheres;
}TGBND;
typedef struct tgelp
{
	char sign[5];
	int length;
	float x,y,z,dx,dy,dz;
}TGELP;
//---------------------------------------------------------------------------
class TIMDFile
{
  private:
    char* buf;
    int size;
    int index;
    TGMAT* gmat;
    TGVTX* gvtx;
	TGREF gref;
	TGBND gbnd;
	TGELP* gelp;
  public:
    int gvtx_count;
    int gmat_count;
    TIMDFile();
    ~TIMDFile();
    void Open(char* file_name);
    TGVTX* GetGVTX(int i);
    TGMAT* GetGMAT(int i);
	TGREF* GetGREF();
	TGBND* GetGBND();
	TGELP* GetGELP();
	int AllVertexCount();
	int AllFacesCount();
};
//---------------------------------------------------------------------------
#endif