/**********************************************************************
 *<
	FILE: objimp.cpp

	DESCRIPTION:  Wavefront .OBJ file import module

	CREATED BY: Don Brittain and Tom Hudson

	HISTORY: created 30 December 1994

 *>	Copyright (c) 1994, All Rights Reserved.
 **********************************************************************/

#include "Max.h"
#include <stdio.h>
#include <direct.h>
#include <commdlg.h>
#include "objimp.h"
#include "objires.h"
#include <dummy.h>
#include <icollision.h>
#include <plugapi.h>
#include <iparamb2.h>
#include <simpobj.h>

#define IDS_DB_POINT                    4
#define PB_RADIUS						0
#define PB_TI_POS						1

// The file stream


HINSTANCE hInstance;

//---------------------------------------------------------------------------
TCHAR *GetString(int id)
	{
	static TCHAR buf[256];
	if (hInstance)
		return LoadString(hInstance, id, buf, sizeof(buf)) ? buf : NULL;
	return NULL;
	}
//---------------------------------------------------------------------------
// Jaguar interface code
BOOL WINAPI DllMain(HINSTANCE hinstDLL,ULONG fdwReason,LPVOID lpvReserved) {
	hInstance = hinstDLL;
	switch(fdwReason) {
		case DLL_PROCESS_ATTACH:
			//MessageBox(NULL,L"OBJIMP.DLL: DllMain",L"OBJIMP",MB_OK);
			break;
		case DLL_THREAD_ATTACH:
			break;
		case DLL_THREAD_DETACH:
			break;
		case DLL_PROCESS_DETACH:
			break;
		}
	return(TRUE);
	}
//---------------------------------------------------------------------------
//------------------------------------------------------
class OBJClassDesc:public ClassDesc {
	public:
	int 			IsPublic() { return 1; }
	void *			Create(BOOL loading = FALSE) { return new OBJImport; }
	const TCHAR *	ClassName() { return GetString(IDS_TH_WAVEFRONT); }
	SClass_ID		SuperClassID() { return SCENE_IMPORT_CLASS_ID; }
	Class_ID		ClassID() { return Class_ID(1,0); }
	const TCHAR* 	Category() { return GetString(IDS_TH_SCENEIMPORT);  }
	};
//---------------------------------------------------------------------------
static OBJClassDesc OBJDesc;
//------------------------------------------------------
// This is the interface to Jaguar:
//------------------------------------------------------
__declspec( dllexport ) const TCHAR *
LibDescription() { return GetString(IDS_TH_OBJIMPORTDLL); }
//---------------------------------------------------------------------------
__declspec( dllexport ) int
LibNumberClasses() { return 1; }
//---------------------------------------------------------------------------
__declspec( dllexport ) ClassDesc *
LibClassDesc(int i) {
	switch(i) {
		case 0: return &OBJDesc; break;
		default: return 0; break;
		}
	}
//---------------------------------------------------------------------------
// Return version so can detect obsolete DLLs
__declspec( dllexport ) ULONG 
LibVersion() { return VERSION_3DSMAX; }
//---------------------------------------------------------------------------
// Let the plug-in register itself for deferred loading
__declspec( dllexport ) ULONG CanAutoDefer()
{
	return 1;
}
//---------------------------------------------------------------------------
//
// .OBJ import module functions follow:
//
//---------------------------------------------------------------------------
OBJImport::OBJImport() {
	fnew_scene=false;
	fmaterial=true;
	frefpoint=true;
	fgbnd=false;
	fgelp=false;
	}
//---------------------------------------------------------------------------
OBJImport::~OBJImport() {
	}
//---------------------------------------------------------------------------
int
OBJImport::ExtCount() {
	return 1;
	}
//---------------------------------------------------------------------------
const TCHAR *
OBJImport::Ext(int n) {		// Extensions supported for import/export modules
	switch(n) {
		case 0:
			return _T("IMD");
		}
	return _T("");
	}
//---------------------------------------------------------------------------
const TCHAR *
OBJImport::LongDesc() {			// Long ASCII description (i.e. "Targa 2.0 Image File")
	return GetString(IDS_TH_WAVEFRONTOBJFILE);
	}
//---------------------------------------------------------------------------
const TCHAR *
OBJImport::ShortDesc() {			// Short ASCII description (i.e. "Targa")
	return GetString(IDS_TH_WAVEFRONT);
	}
//---------------------------------------------------------------------------
const TCHAR *
OBJImport::AuthorName() {			// ASCII Author name
	return GetString(IDS_TH_DON_AND_TOM);
	}
//---------------------------------------------------------------------------
const TCHAR *
OBJImport::CopyrightMessage() {	// ASCII Copyright message
	return GetString(IDS_TH_COPYRIGHT_YOST_GROUP);
	}
//---------------------------------------------------------------------------
const TCHAR *
OBJImport::OtherMessage1() {		// Other message #1
	return _T("");
	}
//---------------------------------------------------------------------------
const TCHAR *
OBJImport::OtherMessage2() {		// Other message #2
	return _T("");
	}
//---------------------------------------------------------------------------
unsigned int
OBJImport::Version() {				// Version number * 100 (i.e. v3.01 = 301)
	return 100;
	}
//---------------------------------------------------------------------------
void
OBJImport::ShowAbout(HWND hWnd) {			// Optional
 	}

//---------------------------------------------------------------------------
static INT_PTR CALLBACK ExportDlgProc(HWND hWnd, UINT msg,
	WPARAM wParam, LPARAM lParam)
{
	OBJImport *imp = (OBJImport*)GetWindowLongPtr(hWnd,GWLP_USERDATA); 
	switch (msg) {
	case WM_INITDIALOG:
		imp=(OBJImport*)lParam;
		SetWindowLongPtr(hWnd,GWLP_USERDATA,lParam); 
		CenterWindow(hWnd, GetParent(hWnd)); 
		CheckDlgButton(hWnd, IDC_NEW_SCENE, imp->fnew_scene); 
		CheckDlgButton(hWnd, IDC_MATERIAL, imp->fmaterial); 
		CheckDlgButton(hWnd, IDC_REFPOINT, imp->frefpoint);
		CheckDlgButton(hWnd, IDC_GBND, imp->fgbnd); 
		CheckDlgButton(hWnd, IDC_GELP, imp->fgelp); 
		break;
	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDOK:
			imp->fnew_scene=IsDlgButtonChecked(hWnd, IDC_NEW_SCENE); 
			imp->fmaterial=IsDlgButtonChecked(hWnd, IDC_MATERIAL); 
			imp->frefpoint=IsDlgButtonChecked(hWnd, IDC_REFPOINT); 
			imp->fgbnd=IsDlgButtonChecked(hWnd, IDC_GBND); 
			imp->fgelp=IsDlgButtonChecked(hWnd, IDC_GELP); 
			EndDialog(hWnd, 1);
			break;
		case IDCANCEL:
			EndDialog(hWnd, 0);
			break;
		}
		break;
		default:
			return FALSE;
	}
	return TRUE;
}       
//---------------------------------------------------------------------------
void OBJImport::ImportGMAT(TIMDFile* imd,StdMat** mtl,Interface *gi)
{
	for(int j=0;j<imd->gmat_count;j++)
	{
	  mtl[j]=NewDefaultStdMat();
	  mtl[j]->SetName(imd->GetGMAT(j)->material_name);
	  for(int k=0;k<imd->GetGMAT(j)->texture_count;k++)
	  {
	    BitmapTex* tex=(BitmapTex*)NewDefaultBitmapTex();
	    char str[500];
	    sprintf(str,"C:\\3DStudioMax\\Images\\%s",imd->GetGMAT(j)->textures[k].file_name);
	    tex->SetMapName(str);
	    tex->GetUVGen()->SetCoordMapping(UVMAP_EXPLICIT);
    	mtl[j]->SetSubTexmap(imd->GetGMAT(j)->textures[k].type,tex);
	    mtl[j]->SetTexmapAmt(imd->GetGMAT(j)->textures[k].type,imd->GetGMAT(j)->textures[k].koeff,0);
	    mtl[j]->EnableMap(imd->GetGMAT(j)->textures[k].type,TRUE);
	  }
	  mtl[j]->LockAmbDiffTex(false);
	  Color c;
	  c.r=imd->GetGMAT(j)->koeff.Ambient.x;
	  c.g=imd->GetGMAT(j)->koeff.Ambient.y;
	  c.b=imd->GetGMAT(j)->koeff.Ambient.z;
	  mtl[j]->SetAmbient(c,gi->GetTime());
	  c.r=imd->GetGMAT(j)->koeff.Diffuse.x;
	  c.g=imd->GetGMAT(j)->koeff.Diffuse.y;
	  c.b=imd->GetGMAT(j)->koeff.Diffuse.z;
	  mtl[j]->SetDiffuse(c,gi->GetTime());
	  c.r=imd->GetGMAT(j)->koeff.Specular.x;
	  c.g=imd->GetGMAT(j)->koeff.Specular.y;
	  c.b=imd->GetGMAT(j)->koeff.Specular.z;
	  mtl[j]->SetSpecular(c,gi->GetTime());
	  c.r=imd->GetGMAT(j)->koeff.Filter.x;
	  c.g=imd->GetGMAT(j)->koeff.Filter.y;
	  c.b=imd->GetGMAT(j)->koeff.Filter.z;
	  mtl[j]->SetFilter(c,gi->GetTime());
	}
}
//---------------------------------------------------------------------------
void OBJImport::ImportGVTX(TIMDFile* imd,StdMat** mtl,ImpInterface *i)
{
	for(int ij=0;ij<imd->gvtx_count;ij++)
	{
	  char str[10];
	  TriObject *object = CreateNewTriObject();
	  object->mesh.setNumVerts(imd->GetGVTX(ij)->point_count);
	  object->mesh.setNumTVerts(imd->GetGVTX(ij)->point_count);
	  object->mesh.setNumFaces(imd->GetGVTX(ij)->polygon_count);
	  object->mesh.setNumTVFaces(imd->GetGVTX(ij)->polygon_count);
	  int vindex=0;
	  int pindex=0;
  	  Point3 vx;
	  ImpNode *node = i->CreateNode();
	  node->Reference(object);
	  i->AddNodeToScene(node);
	  INode* realINode=node->GetINode();
      for(int k=0;k<imd->GetGVTX(ij)->point_count;k++)
	  {
			vx.x=imd->GetGVTX(ij)->points[k].x;
			vx.y=imd->GetGVTX(ij)->points[k].y;
			vx.z=imd->GetGVTX(ij)->points[k].z;
			object->mesh.setVert(k,vx);
			vx.x=imd->GetGVTX(ij)->points[k].a;
			vx.y=1-imd->GetGVTX(ij)->points[k].b;
			vx.z=(float)0.1;
			object->mesh.setTVert(k,vx);
			vx.x=imd->GetGVTX(ij)->normals[k].x;
			vx.y=imd->GetGVTX(ij)->normals[k].y;
			vx.z=imd->GetGVTX(ij)->normals[k].z;
			object->mesh.setNormal(k,vx);
	  }
	  for(int k=0;k<imd->GetGVTX(ij)->polygon_count;k++)
	  {
			Face* f=&(object->mesh.faces[k]);
			f->v[0]=imd->GetGVTX(ij)->polygons[k].a;
			f->v[1]=imd->GetGVTX(ij)->polygons[k].b;
			f->v[2]=imd->GetGVTX(ij)->polygons[k].c;
			TVFace* vf=&(object->mesh.tvFace[k]);
			vf->t[0]=imd->GetGVTX(ij)->polygons[k].a;
			vf->t[1]=imd->GetGVTX(ij)->polygons[k].b;
			vf->t[2]=imd->GetGVTX(ij)->polygons[k].c;
	  }
	  if(mtl!=0)realINode->SetMtl(mtl[imd->GetGVTX(ij)->material_index]);
	  sprintf(str,"OBJ%d",ij);
	  node->SetName(str);
	}
}
//---------------------------------------------------------------------------
void OBJImport::ImportGREF(TIMDFile* imd,Interface *gi,ImpInterface *i)
{
	Matrix3 tm,tm1;
	TGREF* ref=imd->GetGREF();
	for(int j=0;j<ref->ref_count;j++)
	{
//   Dummy
/*		TREFPoint* rp=&ref->refs[j];
		DummyObject* dummy= new DummyObject();
		Point3 vx1,vx2;
		Box3* box;
		vx1.x=-rp->x/2;
		vx1.y=0;
		vx1.z=-rp->z/2;
		vx2.x=rp->x/2;
		vx2.y=rp->y;
		vx2.z=rp->z/2;
		box=new Box3(vx1,vx2);
		dummy->SetBox(*box);
		ImpNode *Node = i->CreateNode();
		Node->Reference(dummy);
		Node->SetName(rp->name);
		i->AddNodeToScene(Node);
		tm.IdentityMatrix();
		tm.Set(Point3(ref->refs[j].matrix[0][0],
					  ref->refs[j].matrix[0][1],
					  ref->refs[j].matrix[0][2]),
			   Point3(ref->refs[j].matrix[1][0],
					  ref->refs[j].matrix[1][1],
					  ref->refs[j].matrix[1][2]),
			   Point3(ref->refs[j].matrix[2][0],
					  ref->refs[j].matrix[2][1],
					  ref->refs[j].matrix[2][2]),
			   Point3(ref->refs[j].matrix[3][0],
					  ref->refs[j].matrix[3][1],
					  ref->refs[j].matrix[3][2]));
		Node->SetTransform(gi->GetTime(),tm);*/

//   PointHelp
   	    tm.IdentityMatrix();
		float size=1;
	    HelperObject * c=(HelperObject*)CreateInstance(0x000050,Class_ID(POINTHELP_CLASS_ID,0));
		IParamBlock2* a=(IParamBlock2*)c->GetParamBlock();
		a->SetValue(pointobj_centermarker,gi->GetTime(),TRUE);
		a->SetValue(pointobj_axistripod, gi->GetTime(), TRUE);
		a->SetValue(pointobj_cross, gi->GetTime(), TRUE);
		a->SetValue(pointobj_box, gi->GetTime(), TRUE);
		a->SetValue(pointobj_size, gi->GetTime(),size);
		ImpNode *Node = i->CreateNode();
		Node->Reference(c);
		i->AddNodeToScene(Node);
		Point3 vec=Point3(0.0,0.25,0.0);
		tm.Translate(vec);
		tm1.IdentityMatrix();
		vec=Point3(ref->refs[j].x*2,ref->refs[j].y*2,ref->refs[j].z*2);
		tm1.Scale(vec);
		tm*=tm1;
		tm1.IdentityMatrix();
		tm1.Set(Point3(ref->refs[j].matrix[0][0],
					  ref->refs[j].matrix[0][1],
					  ref->refs[j].matrix[0][2]),
			   Point3(ref->refs[j].matrix[1][0],
					  ref->refs[j].matrix[1][1],
					  ref->refs[j].matrix[1][2]),
			   Point3(ref->refs[j].matrix[2][0],
					  ref->refs[j].matrix[2][1],
					  ref->refs[j].matrix[2][2]),
			   Point3(ref->refs[j].matrix[3][0],
					  ref->refs[j].matrix[3][1],
					  ref->refs[j].matrix[3][2]));
		tm*=tm1;
		Node->SetTransform(gi->GetTime(),tm);
		Node->SetName(ref->refs[j].name);
	}
}
//---------------------------------------------------------------------------
void OBJImport::ImportGBND(TIMDFile* imd,Interface *gi,ImpInterface *i)
{
	Matrix3 tm;
	TGBND* gbnd=imd->GetGBND();
	GenSphere * c=(GenSphere*)CreateInstance(GEOMOBJECT_CLASS_ID,Class_ID(SPHERE_CLASS_ID,0));
	IParamBlock* a=(IParamBlock*)c->GetParamBlock();
	a->SetValue(PB_RADIUS,gi->GetTime(),gbnd->mainsphera.r);
	ImpNode *Node=i->CreateNode();
	Node->Reference((GenSphere*)c);
	i->AddNodeToScene(Node);
	tm.IdentityMatrix();
	tm.Translate(Point3(gbnd->mainsphera.x,gbnd->mainsphera.y,gbnd->mainsphera.z));
	Node->SetTransform(gi->GetTime(),tm);
	Node->SetName("GBND");
	for(int j=0;j<gbnd->count;j++)
	{
		c=(GenSphere*)CreateInstance(GEOMOBJECT_CLASS_ID,Class_ID(SPHERE_CLASS_ID,0));
		a=(IParamBlock*)c->GetParamBlock();
		a->SetValue(PB_RADIUS,gi->GetTime(),gbnd->spheres[j].r);
		Node=i->CreateNode();
		Node->Reference((GenSphere*)c);
	 	i->AddNodeToScene(Node);
		tm.IdentityMatrix();
		tm.Translate(Point3(gbnd->spheres[j].x,gbnd->spheres[j].y,gbnd->spheres[j].z));
		Node->SetTransform(gi->GetTime(),tm);
		Node->SetName("GBND");
	}
}
//---------------------------------------------------------------------------
void OBJImport::ImportGELP(TIMDFile* imd,Interface *gi,ImpInterface *i)
{
	Matrix3 tm;
	TGELP* gelp=imd->GetGELP();
	if(gelp!=0)
	{
		GenSphere * c=(GenSphere*)CreateInstance(GEOMOBJECT_CLASS_ID,Class_ID(SPHERE_CLASS_ID,0));
		IParamBlock* a=(IParamBlock*)c->GetParamBlock();
		float size=1;
		a->SetValue(PB_RADIUS,gi->GetTime(),size);
		ImpNode *Node = i->CreateNode();
		Node->Reference((GenSphere*)c);
	 	i->AddNodeToScene(Node);
		tm.IdentityMatrix();
		tm.Scale(Point3(gelp->dx,gelp->dy,gelp->dz));
		tm.Translate(Point3(gelp->x,gelp->y,gelp->z));
		Node->SetTransform(gi->GetTime(),tm);
		Node->SetName("GELP");
	}
}
//---------------------------------------------------------------------------
int
OBJImport::DoImport(const TCHAR *filename,ImpInterface *i,Interface *gi, BOOL suppressPrompts) 
{
	StdMat** mtl=0;
	TIMDFile* imd;
	if (!DialogBoxParam(hInstance, MAKEINTRESOURCE(IDD_IMDIMPORT_DLG),
		gi->GetMAXHWnd(), ExportDlgProc, (LPARAM)this)) 
	{
		return 1;
	}
	if(fnew_scene)i->NewScene();
	imd=new TIMDFile;
	imd->Open((char*)filename);
	if(fmaterial)
	{
		mtl=new StdMat*[imd->gmat_count];
		ImportGMAT(imd,mtl,gi);
	}
	ImportGVTX(imd,mtl,i);
	if(frefpoint)ImportGREF(imd,gi,i);
	if(fgbnd)ImportGBND(imd,gi,i);
	if(fgelp)ImportGELP(imd,gi,i);
	if(mtl!=0)delete mtl;
	delete imd;
	i->RedrawViews();
	return 1;
}
//---------------------------------------------------------------------------

